function [errboundpoly,errboundsplines,errNumPoly,errNumNewt,errNumSplines]=...
                                  aufg02error(intval,n,func,funcLip,funcAblnp1,plotten)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Funktionsweise:   Fr eine Funktion func werden auf dem Intervall intval
%                   Interpolationen mittels polyfit/polyval,
%                   Newtoninterpolation und Splines(3,2) fr n+1
%                   Sttzstellen erstellt, die quidistant oder beinahe quidistant
%                   leicht nach links oder rechts verschoben liegen (s.
%                   Abschnitt Vorbereitungen).
%                   Es werden Schtzungen fr theoretische Fehlerschranken fr die
%                   Polynom- und Splineinterpolation berechnet sowie ggf.
%                   die Fehlerkurven erstellt und geplottet.
% Eingabe:
% --------
% intval:           Intervall, auf dem sich alles abspielt
% n:                Anzahl der Subintervalle
% func:             function handle fr die Funktion, die Interpoliert
%                   werden soll
% funcLip:          Schtzung fr die Lipschitzkonstante der dritten
%                   Ableitung von func fr das Intervall intval.
%                   Diese wird fr die Fehlerabschtzung mittels
%                   Splineinterpolation bentigt (s. Ungleichung 2.26 im
%                   Skript)
% funcAblnp1:       Schtzung fr die Inf-Norm der n+1-ten Ableitung von
%                   func auf intval. Diese wird fr die Fehlerabschtzung
%                   der Polynominterpolation bentigt (s. Ungleichung
%                   (2.11) im Skript)
% plotten:          flag, ob die Fehlerkurven geplottet werden sollen
%                   1=ja, 0=nein
%
% Ausgabe:
% --------
% errboundpoly:     Schtzung der theoretischen Fehlerschranke fr die
%                   Polynominterpolation
% errboundsplines:  Schtzung der theoretischen Fehlerschranke fr die
%                   Splineinterpolation
% errNumPoly:       betragsmaximaler Fehler der Polynominterpolation
%                   mittels der MatLab-Funktionen polyfit und polyval
% errNumNewt:       betragsmaximaler Fehler der Polynominterpolation
%                   mittels Newtoninterpolation
% errNumSplines:    betragsmaximaler Fehler der Polynominterpolation
%                   mittels MatLab-Funktionen spline und ppval
%
% v1.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Vorbereitungen
% --------------

% Intervallbreite
intwidth=intval(2)-intval(1);

% Anzahl der Auswertungen fr das Plotten
linpoints=400;

% erzeuge quidistante Knoten
xaequi=linspace(intval(1),intval(2),n+1);
% Breite eines der quidistanten Teilintervalle
laequi=intwidth/n;
% erzeuge zwei neue Zerlegungen, indem um laequi/2 die inneren Punkte
% einmal nach links und einmal nach rechts verschoben werden
xleft=xaequi; xright=xaequi;
xleft(2:end-1)=xleft(2:end-1)-laequi/2;
xright(2:end-1)=xright(2:end-1)+laequi/2;
x_aequi_l_r=[xaequi;xleft;xright];
% das grte Teilintervall ist also gegeben durch
subintmaxlr=1.5*laequi;

func_aequi_l_r=zeros(3,n+1);
poly_aequi_l_r=zeros(3,n+1);
xlin=linspace(intval(1),intval(2),linpoints);


%% Interpolation
% -------------

% Interpolation mithilfe der MatLab-Funktionen polyfit und polyval
polyvals_aequi_l_r=zeros(3,linpoints);
for i=1:3
    % zu interpolierende Funktion an den jeweiligen Knoten ausgewertet
    func_aequi_l_r(i,:)=feval(func,x_aequi_l_r(i,:));
    % bestimme Koeffizienten des jeweiligen Interpolationspolynoms mittels polyfit
    poly_aequi_l_r(i,:)=polyfit(x_aequi_l_r(i,:),func_aequi_l_r(i,:),n);
    % werte aus
    polyvals_aequi_l_r(i,:)=polyval(poly_aequi_l_r(i,:),xlin);
end

% bestimme die Dividierten Differenzen zur Interpolation mit Newton
% nach einer leicht erweiterten Variante des Algorithmus 2.14 des Skriptes
h=zeros(3,n+1); p=zeros(3,n+1);
for k = 1:n+1
   h(:,k) = func_aequi_l_r(:,k);
   if k > 1
      for i = k-1:-1:1
         h(:,i) = (h(:,i+1)-h(:,i))./(x_aequi_l_r(:,k)-x_aequi_l_r(:,i));
      end
   end
   p(:,k) = h(:,1);
end

% Horner-Schema hnliches Vorgehen (vgl. Alg. 2.15) fr die Auswertung des
% Newtoninterpolationspolynomes
pNvals_aequi_l_r = p(:,n+1)*ones(1,linpoints);
x3linspace=[xlin;xlin;xlin];
for i = n:-1:1
    pNvals_aequi_l_r = pNvals_aequi_l_r.*(x3linspace-x_aequi_l_r(:,i)*ones(1,linpoints))+p(:,i)*ones(1,linpoints);
end

% spline-Interpolation aus der Klasse (3,2) mittels der MatLab-Funktionen
% spline und ppval
splines_aequi_l_r=cell(1,3);
splinevals_aequi_l_r=zeros(3,linpoints);
for i=1:3
    % erstelle den Spline
    splines_aequi_l_r{i}=spline(x_aequi_l_r(i,:),func_aequi_l_r(i,:));
    % werte ihn aus
    splinevals_aequi_l_r(i,:)=ppval(splines_aequi_l_r{i},xlin);
end


%% Schtzungen fr die Fehlerschranken
% -----------------------------------

% fr die Polynominterpolation gehen wir nach Bemerkung 2.22 des Skriptes
% vor
% wir bestimmen zunchst das w, respektive eine Nherung hierfr
w_aequi_l_r=1; 
for i=1:n+1
    w_aequi_l_r=w_aequi_l_r.*(x3linspace-x_aequi_l_r(:,i)*ones(1,linpoints));
end
w=max(abs(w_aequi_l_r'));
% und damit die Schranke (2.11) im Skript
errboundpoly=feval(funcAblnp1,n,intval)/factorial(n+1)*w;
% fr groe n wird die Fakultt den Ausdruck dominieren, wir schneiden
% daher bei eps ab, um fr einen Plot die Gren nicht zu weit auseinander
% zu ziehen
errboundpoly=max(errboundpoly,eps);

% fr die Splines nutzen wir Satz 2.29, Ungleichung (2.26)
% fr die quidistanten Knoten haben wir eine Subintervallnge von laequi
errboundsplines=zeros(1,3);
errboundsplines(1)=9/8*feval(funcLip,intval)*(laequi^4);
% fr die zwei Zerlegungen, in denen nach links respektive nach rechts die
% inneren Knoten verschoben wurden, ist die maximale Subintervallnge
% gegeben ber subintmaxlr
errboundsplines(2)=9/8*feval(funcLip,intval)*(subintmaxlr^4);
errboundsplines(3)=errboundsplines(2);
% sicherheitshalber schneiden wir hier auch bei eps ab
errboundsplines=max(errboundsplines,eps);


%% echten numerischen Fehler abschtzen
% ------------------------------------
funcwerte=feval(func,xlin);
errPoly_aequi_l_r=zeros(3,linpoints);
errNewt_aequi_l_r=zeros(3,linpoints);
errSplines_aequi_l_r=zeros(3,linpoints);

for i=1:3
    errPoly_aequi_l_r(i,:)=funcwerte-polyvals_aequi_l_r(i,:);
    errNewt_aequi_l_r(i,:)=funcwerte-pNvals_aequi_l_r(i,:);
    errSplines_aequi_l_r(i,:)=funcwerte-splinevals_aequi_l_r(i,:);
end

% bounds fr die echten Fehler zurckgeben
errNumPoly=max(abs(errPoly_aequi_l_r)');
errNumNewt=max(abs(errNewt_aequi_l_r)');
errNumSplines=max(abs(errSplines_aequi_l_r)');


%% Fehler plotten
% --------------
beschriftungen={{'quidistant'},{'Linksshift'},{'Rechtsshift'}};
verfahren={{'polyfit'},{'Newton'},{'Spline(3,2)'}};
if plotten
% plotten der Fehlerfunktion fr das polyfit-Interpolationspolynom
for i=1:3
    subplot(3,3,i), plot(xlin,errPoly_aequi_l_r(i,:),'r-'), title(strcat(beschriftungen{i},', ',verfahren{1}))
    hold on
    subplot(3,3,i), plot(x_aequi_l_r(i,:),zeros(1,n+1),'ob'), l=legend('Fehlerkurve','Interpolationsknoten');
    set(l,'Location','SouthOutside');
end

% plotten des Fehlers der Newton Interpolation, dieser sollte theoretisch gleich dem
% Fehler der Interpolation mittels polyfit sein (in exakter Rechnung wird
% ja das gleiche Interpolationspolynom erstellt!)
for i=1:3
    subplot(3,3,i+3), plot(xlin,errNewt_aequi_l_r(i,:),'r-'), title(strcat(beschriftungen{i},', ',verfahren{2}))
    hold on
    subplot(3,3,i+3), 
    plot(x_aequi_l_r(i,:),zeros(1,n+1),'ob'), l=legend('Fehlerkurve','Interpolationsknoten');
    set(l,'Location','SouthOutside');
end

% plotten des Fehlers der spline-Interpolation
for i=1:3
    subplot(3,3,i+6), plot(xlin,errSplines_aequi_l_r(i,:),'r-'), title(strcat(beschriftungen{i},', ',verfahren{3}))
    hold on
    subplot(3,3,i+6), plot(x_aequi_l_r(i,:),zeros(1,n+1),'ob'), l=legend('Fehlerkurve','Interpolationsknoten');
    set(l,'Location','SouthOutside');
end
end % if plotten